@@ -69,11 +69,8 @@ ast_struct! {
     /// [`Attribute::parse_outer`]: #method.parse_outer
     /// [`Attribute::parse_inner`]: #method.parse_inner
     ///
-    /// ```
-    /// #[macro_use]
-    /// extern crate syn;
-    ///
-    /// use syn::{Attribute, Ident, Result};
+    /// ```edition2018
+    /// use syn::{Attribute, Ident, Result, Token};
     /// use syn::parse::{Parse, ParseStream};
     ///
     /// // Parses a unit struct with attributes.
@@ -97,8 +94,6 @@ ast_struct! {
     ///         })
     ///     }
     /// }
-    /// #
-    /// # fn main() {}
     /// ```
     pub struct Attribute #manual_extra_traits {
         pub pound_token: Token![#],
@@ -138,8 +133,8 @@ impl Hash for Attribute {
 }
 
 impl Attribute {
-    /// Parses the tokens after the path as a [`Meta`](enum.Meta.html) if
-    /// possible.
+    /// Parses the content of the attribute, consisting of the path and tts, as
+    /// a [`Meta`](enum.Meta.html) if possible.
     ///
     /// Deprecated; use `parse_meta` instead.
     #[doc(hidden)]
@@ -179,8 +174,8 @@ impl Attribute {
         }
     }
 
-    /// Parses the tokens after the path as a [`Meta`](enum.Meta.html) if
-    /// possible.
+    /// Parses the content of the attribute, consisting of the path and tts, as
+    /// a [`Meta`](enum.Meta.html) if possible.
     #[cfg(feature = "parsing")]
     pub fn parse_meta(&self) -> Result<Meta> {
         if let Some(ref colon) = self.path.leading_colon {
@@ -475,7 +470,7 @@ ast_enum_of_structs! {
 /// For example if we are developing an attribute macro that is intended to be
 /// invoked on function items as follows:
 ///
-/// ```rust
+/// ```edition2018
 /// # const IGNORE: &str = stringify! {
 /// #[my_attribute(path = "/v1/refresh")]
 /// # };
@@ -487,14 +482,11 @@ ast_enum_of_structs! {
 /// The implementation of this macro would want to parse its attribute arguments
 /// as type `AttributeArgs`.
 ///
-/// ```rust
-/// #[macro_use]
-/// extern crate syn;
-///
+/// ```edition2018
 /// extern crate proc_macro;
 ///
 /// use proc_macro::TokenStream;
-/// use syn::{AttributeArgs, ItemFn};
+/// use syn::{parse_macro_input, AttributeArgs, ItemFn};
 ///
 /// # const IGNORE: &str = stringify! {
 /// #[proc_macro_attribute]
@@ -506,8 +498,6 @@ ast_enum_of_structs! {
 ///     /* ... */
 /// #   "".parse().unwrap()
 /// }
-/// #
-/// # fn main() {}
 /// ```
 pub type AttributeArgs = Vec<NestedMeta>;
 
@@ -610,11 +600,9 @@ pub mod parsing {
 
     impl Parse for NestedMeta {
         fn parse(input: ParseStream) -> Result<Self> {
-            let ahead = input.fork();
-
-            if ahead.peek(Lit) && !(ahead.peek(LitBool) && ahead.peek2(Token![=])) {
+            if input.peek(Lit) && !(input.peek(LitBool) && input.peek2(Token![=])) {
                 input.parse().map(NestedMeta::Literal)
-            } else if ahead.call(Ident::parse_any).is_ok() {
+            } else if input.peek(Ident::peek_any) {
                 input.parse().map(NestedMeta::Meta)
             } else {
                 Err(input.error("expected identifier or literal"))
