@@ -25,14 +25,11 @@
 //!
 //! [`parse_macro_input!`]: ../macro.parse_macro_input.html
 //!
-//! ```
-//! #[macro_use]
-//! extern crate syn;
-//!
+//! ```edition2018
 //! extern crate proc_macro;
 //!
 //! use proc_macro::TokenStream;
-//! use syn::{token, Field, Ident, Result};
+//! use syn::{braced, parse_macro_input, token, Field, Ident, Result, Token};
 //! use syn::parse::{Parse, ParseStream};
 //! use syn::punctuated::Punctuated;
 //!
@@ -90,8 +87,6 @@
 //!     /* ... */
 //! #   "".parse().unwrap()
 //! }
-//! #
-//! # fn main() {}
 //! ```
 //!
 //! # The `syn::parse*` functions
@@ -106,7 +101,7 @@
 //! [`syn::parse_str`]: ../fn.parse_str.html
 //! [`Parse`]: trait.Parse.html
 //!
-//! ```
+//! ```edition2018
 //! use syn::Type;
 //!
 //! # fn run_parser() -> syn::Result<()> {
@@ -137,9 +132,8 @@
 //! The `Parse` trait is not implemented in these cases because there is no good
 //! behavior to consider the default.
 //!
-//! ```compile_fail
+//! ```edition2018,compile_fail
 //! # extern crate proc_macro;
-//! # extern crate syn;
 //! #
 //! # use syn::punctuated::Punctuated;
 //! # use syn::{PathSegment, Result, Token};
@@ -160,45 +154,43 @@
 //!
 //! [`Parser`]: trait.Parser.html
 //!
-//! ```
-//! #[macro_use]
-//! extern crate syn;
+//! ```edition2018
+//! extern crate proc_macro;
 //!
-//! extern crate proc_macro2;
-//!
-//! use proc_macro2::TokenStream;
+//! use proc_macro::TokenStream;
 //! use syn::parse::Parser;
 //! use syn::punctuated::Punctuated;
-//! use syn::{Attribute, Expr, PathSegment};
+//! use syn::{Attribute, Expr, PathSegment, Result, Token};
 //!
-//! # fn run_parsers() -> syn::Result<()> {
-//! #     let tokens = TokenStream::new().into();
-//! // Parse a nonempty sequence of path segments separated by `::` punctuation
-//! // with no trailing punctuation.
-//! let parser = Punctuated::<PathSegment, Token![::]>::parse_separated_nonempty;
-//! let path = parser.parse(tokens)?;
+//! fn call_some_parser_methods(input: TokenStream) -> Result<()> {
+//!     // Parse a nonempty sequence of path segments separated by `::` punctuation
+//!     // with no trailing punctuation.
+//!     let tokens = input.clone();
+//!     let parser = Punctuated::<PathSegment, Token![::]>::parse_separated_nonempty;
+//!     let _path = parser.parse(tokens)?;
 //!
-//! #     let tokens = TokenStream::new().into();
-//! // Parse a possibly empty sequence of expressions terminated by commas with
-//! // an optional trailing punctuation.
-//! let parser = Punctuated::<Expr, Token![,]>::parse_terminated;
-//! let args = parser.parse(tokens)?;
+//!     // Parse a possibly empty sequence of expressions terminated by commas with
+//!     // an optional trailing punctuation.
+//!     let tokens = input.clone();
+//!     let parser = Punctuated::<Expr, Token![,]>::parse_terminated;
+//!     let _args = parser.parse(tokens)?;
 //!
-//! #     let tokens = TokenStream::new().into();
-//! // Parse zero or more outer attributes but not inner attributes.
-//! let parser = Attribute::parse_outer;
-//! let attrs = parser.parse(tokens)?;
-//! #
-//! #     Ok(())
-//! # }
-//! #
-//! # fn main() {}
+//!     // Parse zero or more outer attributes but not inner attributes.
+//!     let tokens = input.clone();
+//!     let parser = Attribute::parse_outer;
+//!     let _attrs = parser.parse(tokens)?;
+//!
+//!     Ok(())
+//! }
 //! ```
 //!
 //! ---
 //!
 //! *This module is available if Syn is built with the `"parsing"` feature.*
 
+#[path = "discouraged.rs"]
+pub mod discouraged;
+
 use std::cell::Cell;
 use std::fmt::{self, Debug, Display};
 use std::marker::PhantomData;
@@ -245,6 +237,19 @@ pub type ParseStream<'a> = &'a ParseBuffer<'a>;
 ///
 /// `ParseStream` is the input type for all parser functions in Syn. They have
 /// the signature `fn(ParseStream) -> Result<T>`.
+///
+/// ## Calling a parser function
+///
+/// There is no public way to construct a `ParseBuffer`. Instead, if you are
+/// looking to invoke a parser function that requires `ParseStream` as input,
+/// you will need to go through one of the public parsing entry points.
+///
+/// - The [`parse_macro_input!`] macro if parsing input of a procedural macro;
+/// - One of [the `syn::parse*` functions][syn-parse]; or
+/// - A method of the [`Parser`] trait.
+///
+/// [`parse_macro_input!`]: ../macro.parse_macro_input.html
+/// [syn-parse]: index.html#the-synparse-functions
 pub struct ParseBuffer<'a> {
     scope: Span,
     // Instead of Cell<Cursor<'a>> so that ParseBuffer<'a> is covariant in 'a.
@@ -291,10 +296,7 @@ impl<'a> Debug for ParseBuffer<'a> {
 ///
 /// # Example
 ///
-/// ```
-/// # extern crate proc_macro2;
-/// # extern crate syn;
-/// #
+/// ```edition2018
 /// use proc_macro2::TokenTree;
 /// use syn::Result;
 /// use syn::parse::ParseStream;
@@ -306,8 +308,8 @@ impl<'a> Debug for ParseBuffer<'a> {
 ///     input.step(|cursor| {
 ///         let mut rest = *cursor;
 ///         while let Some((tt, next)) = rest.token_tree() {
-///             match tt {
-///                 TokenTree::Punct(ref punct) if punct.as_char() == '@' => {
+///             match &tt {
+///                 TokenTree::Punct(punct) if punct.as_char() == '@' => {
 ///                     return Ok(((), next));
 ///                 }
 ///                 _ => rest = next,
@@ -317,7 +319,6 @@ impl<'a> Debug for ParseBuffer<'a> {
 ///     })
 /// }
 /// #
-/// #
 /// # fn remainder_after_skipping_past_next_at(
 /// #     input: ParseStream,
 /// # ) -> Result<proc_macro2::TokenStream> {
@@ -429,11 +430,8 @@ impl<'a> ParseBuffer<'a> {
     ///
     /// [`Attribute::parse_outer`]: ../struct.Attribute.html#method.parse_outer
     ///
-    /// ```
-    /// #[macro_use]
-    /// extern crate syn;
-    ///
-    /// use syn::{Attribute, Ident, Result};
+    /// ```edition2018
+    /// use syn::{Attribute, Ident, Result, Token};
     /// use syn::parse::{Parse, ParseStream};
     ///
     /// // Parses a unit struct with attributes.
@@ -457,8 +455,6 @@ impl<'a> ParseBuffer<'a> {
     ///         })
     ///     }
     /// }
-    /// #
-    /// # fn main() {}
     /// ```
     pub fn call<T>(&self, function: fn(ParseStream) -> Result<T>) -> Result<T> {
         function(self)
@@ -476,7 +472,8 @@ impl<'a> ParseBuffer<'a> {
     ///
     /// - `input.peek(Token![struct])`
     /// - `input.peek(Token![==])`
-    /// - `input.peek(Ident)`
+    /// - `input.peek(Ident)`&emsp;*(does not accept keywords)*
+    /// - `input.peek(Ident::peek_any)`
     /// - `input.peek(Lifetime)`
     /// - `input.peek(token::Brace)`
     ///
@@ -485,11 +482,8 @@ impl<'a> ParseBuffer<'a> {
     /// In this example we finish parsing the list of supertraits when the next
     /// token in the input is either `where` or an opening curly brace.
     ///
-    /// ```
-    /// #[macro_use]
-    /// extern crate syn;
-    ///
-    /// use syn::{token, Generics, Ident, Result, TypeParamBound};
+    /// ```edition2018
+    /// use syn::{braced, token, Generics, Ident, Result, Token, TypeParamBound};
     /// use syn::parse::{Parse, ParseStream};
     /// use syn::punctuated::Punctuated;
     ///
@@ -537,8 +531,6 @@ impl<'a> ParseBuffer<'a> {
     ///         })
     ///     }
     /// }
-    /// #
-    /// # fn main() {}
     /// ```
     pub fn peek<T: Peek>(&self, token: T) -> bool {
         let _ = token;
@@ -557,11 +549,8 @@ impl<'a> ParseBuffer<'a> {
     /// union` and a macro invocation that looks like `union::some_macro! { ...
     /// }`. In other words `union` is a contextual keyword.
     ///
-    /// ```
-    /// #[macro_use]
-    /// extern crate syn;
-    ///
-    /// use syn::{Ident, ItemUnion, Macro, Result};
+    /// ```edition2018
+    /// use syn::{Ident, ItemUnion, Macro, Result, Token};
     /// use syn::parse::{Parse, ParseStream};
     ///
     /// // Parses either a union or a macro invocation.
@@ -581,8 +570,6 @@ impl<'a> ParseBuffer<'a> {
     ///         }
     ///     }
     /// }
-    /// #
-    /// # fn main() {}
     /// ```
     pub fn peek2<T: Peek>(&self, token: T) -> bool {
         let ahead = self.fork();
@@ -603,14 +590,10 @@ impl<'a> ParseBuffer<'a> {
     ///
     /// # Example
     ///
-    /// ```rust
-    /// # #[macro_use]
-    /// # extern crate quote;
+    /// ```edition2018
+    /// # use quote::quote;
     /// #
-    /// #[macro_use]
-    /// extern crate syn;
-    ///
-    /// use syn::{token, Ident, Result, Type};
+    /// use syn::{parenthesized, token, Ident, Result, Token, Type};
     /// use syn::parse::{Parse, ParseStream};
     /// use syn::punctuated::Punctuated;
     ///
@@ -659,11 +642,8 @@ impl<'a> ParseBuffer<'a> {
     ///
     /// # Example
     ///
-    /// ```rust
-    /// #[macro_use]
-    /// extern crate syn;
-    ///
-    /// use syn::{token, Ident, Item, Result};
+    /// ```edition2018
+    /// use syn::{braced, token, Ident, Item, Result, Token};
     /// use syn::parse::{Parse, ParseStream};
     ///
     /// // Parses a Rust `mod m { ... }` containing zero or more items.
@@ -691,8 +671,6 @@ impl<'a> ParseBuffer<'a> {
     ///         })
     ///     }
     /// }
-    /// #
-    /// # fn main() {}
     /// ```
     pub fn is_empty(&self) -> bool {
         self.cursor().eof()
@@ -703,11 +681,8 @@ impl<'a> ParseBuffer<'a> {
     ///
     /// # Example
     ///
-    /// ```
-    /// #[macro_use]
-    /// extern crate syn;
-    ///
-    /// use syn::{ConstParam, Ident, Lifetime, LifetimeDef, Result, TypeParam};
+    /// ```edition2018
+    /// use syn::{ConstParam, Ident, Lifetime, LifetimeDef, Result, Token, TypeParam};
     /// use syn::parse::{Parse, ParseStream};
     ///
     /// // A generic parameter, a single one of the comma-separated elements inside
@@ -741,8 +716,6 @@ impl<'a> ParseBuffer<'a> {
     ///         }
     ///     }
     /// }
-    /// #
-    /// # fn main() {}
     /// ```
     pub fn lookahead1(&self) -> Lookahead1<'a> {
         lookahead::new(self.scope, self.cursor())
@@ -758,7 +731,7 @@ impl<'a> ParseBuffer<'a> {
     /// is if your macro ends up parsing a large amount of content more than
     /// once.
     ///
-    /// ```
+    /// ```edition2018
     /// # use syn::{Expr, Result};
     /// # use syn::parse::ParseStream;
     /// #
@@ -775,9 +748,15 @@ impl<'a> ParseBuffer<'a> {
     /// parse stream. Only use a fork when the amount of work performed against
     /// the fork is small and bounded.
     ///
-    /// For a lower level but occasionally more performant way to perform
-    /// speculative parsing, consider using [`ParseStream::step`] instead.
+    /// When complex speculative parsing against the forked stream is
+    /// unavoidable, use [`parse::discouraged::Speculative`] to advance the
+    /// original stream once the fork's parse is determined to have been
+    /// successful.
     ///
+    /// For a lower level way to perform speculative parsing at the token level,
+    /// consider using [`ParseStream::step`] instead.
+    ///
+    /// [`parse::discouraged::Speculative`]: ./discouraged/trait.Speculative.html
     /// [`ParseStream::step`]: #method.step
     ///
     /// # Example
@@ -795,7 +774,7 @@ impl<'a> ParseBuffer<'a> {
     /// needs to distinguish parentheses that specify visibility restrictions
     /// from parentheses that form part of a tuple type.
     ///
-    /// ```
+    /// ```edition2018
     /// # struct A;
     /// # struct B;
     /// # struct C;
@@ -812,11 +791,8 @@ impl<'a> ParseBuffer<'a> {
     /// parentheses after the `pub` keyword. This is a small bounded amount of
     /// work performed against the forked parse stream.
     ///
-    /// ```
-    /// #[macro_use]
-    /// extern crate syn;
-    ///
-    /// use syn::{token, Ident, Path, Result};
+    /// ```edition2018
+    /// use syn::{parenthesized, token, Ident, Path, Result, Token};
     /// use syn::ext::IdentExt;
     /// use syn::parse::{Parse, ParseStream};
     ///
@@ -870,8 +846,6 @@ impl<'a> ParseBuffer<'a> {
     ///         })
     ///     }
     /// }
-    /// #
-    /// # fn main() {}
     /// ```
     pub fn fork(&self) -> Self {
         ParseBuffer {
@@ -888,11 +862,8 @@ impl<'a> ParseBuffer<'a> {
     ///
     /// # Example
     ///
-    /// ```
-    /// #[macro_use]
-    /// extern crate syn;
-    ///
-    /// use syn::{Expr, Result};
+    /// ```edition2018
+    /// use syn::{Expr, Result, Token};
     /// use syn::parse::{Parse, ParseStream};
     ///
     /// // Some kind of loop: `while` or `for` or `loop`.
@@ -914,8 +885,6 @@ impl<'a> ParseBuffer<'a> {
     ///         }
     ///     }
     /// }
-    /// #
-    /// # fn main() {}
     /// ```
     pub fn error<T: Display>(&self, message: T) -> Error {
         error::new_at(self.scope, self.cursor(), message)
@@ -930,10 +899,7 @@ impl<'a> ParseBuffer<'a> {
     ///
     /// # Example
     ///
-    /// ```
-    /// # extern crate proc_macro2;
-    /// # extern crate syn;
-    /// #
+    /// ```edition2018
     /// use proc_macro2::TokenTree;
     /// use syn::Result;
     /// use syn::parse::ParseStream;
@@ -945,8 +911,8 @@ impl<'a> ParseBuffer<'a> {
     ///     input.step(|cursor| {
     ///         let mut rest = *cursor;
     ///         while let Some((tt, next)) = rest.token_tree() {
-    ///             match tt {
-    ///                 TokenTree::Punct(ref punct) if punct.as_char() == '@' => {
+    ///             match &tt {
+    ///                 TokenTree::Punct(punct) if punct.as_char() == '@' => {
     ///                     return Ok(((), next));
     ///                 }
     ///                 _ => rest = next,
@@ -1093,10 +1059,16 @@ pub trait Parser: Sized {
     type Output;
 
     /// Parse a proc-macro2 token stream into the chosen syntax tree node.
+    ///
+    /// This function will check that the input is fully parsed. If there are
+    /// any unparsed tokens at the end of the stream, an error is returned.
     fn parse2(self, tokens: TokenStream) -> Result<Self::Output>;
 
     /// Parse tokens of source code into the chosen syntax tree node.
     ///
+    /// This function will check that the input is fully parsed. If there are
+    /// any unparsed tokens at the end of the stream, an error is returned.
+    ///
     /// *This method is available if Syn is built with both the `"parsing"` and
     /// `"proc-macro"` features.*
     #[cfg(all(
@@ -1109,6 +1081,9 @@ pub trait Parser: Sized {
 
     /// Parse a string of Rust code into the chosen syntax tree node.
     ///
+    /// This function will check that the input is fully parsed. If there are
+    /// any unparsed tokens at the end of the string, an error is returned.
+    ///
     /// # Hygiene
     ///
     /// Every span in the resulting syntax tree will be set to resolve at the
